package com.gic.udesk.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.udesk.dto.TapdCustomFieldConfig;
import com.gic.udesk.dto.TapdRespCustomFieldDTO;
import com.gic.udesk.dto.TapdRespCustomFieldDataDTO;
import com.gic.udesk.entity.TabUdeskTapdRel;
import com.gic.udesk.service.TapdConfigApiService;
import com.gic.udesk.service.UdeskBaseApiService;
import com.gic.udesk.service.UdeskTapdRelService;
import com.gic.udesk.utils.UdeskConstant;
import com.gic.udesk.utils.UdeskHttpClientUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("tapdConfigApiService")
public class TapdConfigApiServiceImpl implements TapdConfigApiService {
    private static Logger logger = LogManager.getLogger(TapdConfigApiServiceImpl.class);

    @Autowired
    private UdeskBaseApiService udeskBaseApiService;
    @Autowired
    private UdeskTapdRelService udeskTapdRelService;
    /**
     * tapd获取缺陷自定义字段配置
     * 接口路径
     */
    private static final String CUSTOM_FIELDS_SETTINGS = "/bugs/custom_fields_settings";

    @Override
    public ServiceResponse<String> setTapdCustomFieldRelUdesk(String workspaceId) {
        Map<String, Object> param = new HashMap<>(4);
        param.put("workspace_id", workspaceId);
        logger.info("获取TAPD缺陷自定义字段入参为{}", workspaceId);
        String customFieldsStr = UdeskHttpClientUtil.getTapdJsonFromParam(CUSTOM_FIELDS_SETTINGS, param);
        logger.info("获取TAPD缺陷自定义字段返回为{}", customFieldsStr);
        //转为对象
        TapdRespCustomFieldDTO tapdCustomField = JSON.parseObject(customFieldsStr, TapdRespCustomFieldDTO.class);

        if (tapdCustomField.getStatus() == UdeskConstant.TAPD_RESP_SUCCESS_STATUS) {
            //9 : tapd的项目ID和udesk的自定义字段的关联
            udeskTapdRelService.deleteByWorkspaceId(9, workspaceId);
            List<TapdRespCustomFieldDataDTO> customFieldData = tapdCustomField.getData();
            for (TapdRespCustomFieldDataDTO customField : customFieldData) {
                TapdCustomFieldConfig customFieldConfig = customField.getCustomFieldConfig();
                String name = customFieldConfig.getName();
                String custom_field = customFieldConfig.getCustom_field();
                TabUdeskTapdRel tabUdeskTapdRel = null;
                //tapd返回的自定义字段数据，没有不变化的ID，所以用名称来匹配
                //tapd不同项目建立如录入客服这个自定义字段的时候，对应的key是不确定的，所以用名称来匹配，且用反射来获取值
                switch (name) {
                case "提报人":
                    tabUdeskTapdRel = new TabUdeskTapdRel(workspaceId, 9, "getUserName", "客户姓名",
                            "set" + captureName(custom_field), "提报人");
                    break;
                case "录入客服":
                    tabUdeskTapdRel = new TabUdeskTapdRel(workspaceId, 9, "getServiceStaff", "录入客服",
                            "set" + captureName(custom_field), "录入客服");
                    break;
                case "UDESK工单编号":
                    tabUdeskTapdRel = new TabUdeskTapdRel(workspaceId, 9, "getFieldNum", "UDESK工单编号",
                            "set" + captureName(custom_field), "UDESK工单编号");
                    break;
                case "公司信息":
                    tabUdeskTapdRel = new TabUdeskTapdRel(workspaceId, 9, "getOrganizationName", "公司",
                            "set" + captureName(custom_field), "公司信息");
                    break;
                default:
                    break;
                }
                if (tabUdeskTapdRel != null) {
                    udeskTapdRelService.save(tabUdeskTapdRel);
                }
            }
        } else {
            String exceptionRecord = "tapd项目id为" + workspaceId + "查询自定义字段失败，返回为" + customFieldsStr;
            logger.info(exceptionRecord);
            udeskBaseApiService.saveTapdException(exceptionRecord);
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "查询tapd自定义字段失败");
        }
        return ServiceResponse.success();
    }

    /**
     * 首字母大写
     */
    private String captureName(String name) {
        char[] cs = name.toCharArray();
        cs[0] -= 32;
        return String.valueOf(cs);
    }
}
