package com.gic.udesk.service.outer.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;

import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.udesk.dto.customfield.UdeskCustomFieldOptionDTO;
import com.gic.udesk.dto.customfield.UdeskResCustomFieldDTO;
import com.gic.udesk.dto.customfield.UdeskResCustomFieldDataDTO;
import com.gic.udesk.dto.templates.UdeskResTemplatesDTO;
import com.gic.udesk.dto.templates.UdeskResTemplatesDataDTO;
import com.gic.udesk.dto.templates.UdeskTemplatesCustomFieldDTO;
import com.gic.udesk.dto.uploadfile.UploadFileDTO;
import com.gic.udesk.entity.TabUdeskException;
import com.gic.udesk.entity.TabUdeskTapdRel;
import com.gic.udesk.service.UdeskBaseApiService;
import com.gic.udesk.service.UdeskExceptionService;
import com.gic.udesk.service.UdeskTapdRelService;
import com.gic.udesk.utils.UdeskConstant;
import com.gic.udesk.utils.UdeskHttpClientUtil;

@Service("udeskBaseApiService")
public class UdeskBaseApiServiceImpl implements UdeskBaseApiService {
    private static Logger logger = LogManager.getLogger(UdeskBaseApiServiceImpl.class);

    @Autowired
    private UdeskExceptionService udeskExceptionService;
    @Autowired
    private UdeskTapdRelService udeskTapdRelService;

    @Resource(name = "udeskTapdRelMap")
    private Map<String, String> udeskTapdRelMap;
    /**
     * 取工单模板列表
     */
    private final static String UDESK_TEMPLATE = "/tickets/ticket_templates";

    private final static String CUSTOM_FIELD_URL = "/custom_fields";

    private final static String UPLOAD_FILE = "/tickets/upload_file";

    @PostConstruct
    public void init() {
        logger.info("初始化udeskTapdRelMap启动。。。。。。。。。。。。。。。");
        List<TabUdeskTapdRel> list = udeskTapdRelService.list(null);
        if (CollectionUtils.isNotEmpty(list)) {
            logger.info("共有" + list.size() + "条数据要处理");
            for (TabUdeskTapdRel tabUdeskTapdRel : list) {
                Integer relType = tabUdeskTapdRel.getRelType();
                if (relType == 1) {
                    // 1 udesk的业务模块与tapd项目的对应
                    udeskTapdRelMap.put(tabUdeskTapdRel.getUdeskIdKey() + ":" + tabUdeskTapdRel.getUdeskIdValue(),
                            tabUdeskTapdRel.getTapdId());
                    continue;
                } else if (relType == 8) {
                    // 8 udesk状态 -> tapd状态
                    udeskTapdRelMap.put(tabUdeskTapdRel.getUdeskName(), tabUdeskTapdRel.getTapdId());
                    continue;
                } else if (relType == 5 || relType == 4 || relType == 7) {
                    // 4 tapd严重程度->udsek 5 tapd状态  -> udesk状态  7 优先级
                    udeskTapdRelMap.put(tabUdeskTapdRel.getTapdId() + UdeskConstant.MAP_UDESK_ID_VALUE_SUFFIX,
                            tabUdeskTapdRel.getUdeskIdValue());
                    udeskTapdRelMap.put(tabUdeskTapdRel.getTapdId() + UdeskConstant.MAP_UDESK_NAME_SUFFIX,
                            tabUdeskTapdRel.getUdeskName());
                    if (relType == 4) {
                        // udesk严重程度字段名：idvalue -> udsek严重程度CN
                        udeskTapdRelMap.put(tabUdeskTapdRel.getUdeskIdKey() + ":" + tabUdeskTapdRel.getUdeskIdValue(),
                                tabUdeskTapdRel.getUdeskName());
                    }
                    continue;
                } else if (relType == 6) {
                    // 6 客服
                    udeskTapdRelMap.put(UdeskConstant.UDESK_DEFAULT_AGENT, tabUdeskTapdRel.getUdeskIdValue());
                    continue;
                }
                udeskTapdRelMap.put(tabUdeskTapdRel.getUdeskIdKey() + ":" + tabUdeskTapdRel.getUdeskIdValue(),
                        tabUdeskTapdRel.getUdeskName());
            }
            logger.info("初始化udeskTapdRelMap完成：" + JSON.toJSONString(udeskTapdRelMap));
        }

    }

    @Override
    public ServiceResponse<Void> saveUdeskException(String exceptionRecord) {
        TabUdeskException exception = new TabUdeskException();
        exception.setExpType(UdeskConstant.UDESK_EXP_TYPE);
        exception.setExceptionRecord(exceptionRecord);
        udeskExceptionService.save(exception);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveTapdException(String exceptionRecord) {
        TabUdeskException exception = new TabUdeskException();
        exception.setExpType(UdeskConstant.TAPD_EXP_TYPE);
        exception.setExceptionRecord(exceptionRecord);
        udeskExceptionService.save(exception);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<UdeskResCustomFieldDataDTO>> getCustomFieldFromUdeskTemplates(Integer type) {
        boolean isNotRightType = type == null || (type.intValue() != 1 && type.intValue() != 2);
        if (isNotRightType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数错误,数值非法");
        }
        logger.info("udesk工单模板链接");
        String replyRespStr = UdeskHttpClientUtil.getUdeskJsonFromParam(UDESK_TEMPLATE, null);
        logger.info("desk工单模板链接返回为{}", replyRespStr);
        UdeskResTemplatesDTO result = JSON.parseObject(replyRespStr, UdeskResTemplatesDTO.class);
        if (result.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            List<UdeskTemplatesCustomFieldDTO> customFieldList = null;
            for (UdeskResTemplatesDataDTO templatesDataDTO : result.getTemplates()) {
                if (type == 1) {
                    //gic基础业务模板
                    if (templatesDataDTO.getId().intValue() == 97811) {
                        customFieldList = templatesDataDTO.getCustom_fields();
                        break;
                    }
                } else {
                    //应用模板
                    if (templatesDataDTO.getId().intValue() == 99311) {
                        customFieldList = templatesDataDTO.getCustom_fields();
                        break;
                    }
                }
            }
            if (CollectionUtils.isNotEmpty(customFieldList)) {
                List<String> fieldNameList = new ArrayList<>(customFieldList.size());
                for (UdeskTemplatesCustomFieldDTO customFieldDTO : customFieldList) {
                    fieldNameList.add(customFieldDTO.getField_name());
                }
                return ServiceResponse.success(queryUdeskCustomField(fieldNameList));
            }
        } else {
            String exceptionRecord = "udesk工单模板查询异常，返回为" + replyRespStr;
            logger.info(exceptionRecord);
            this.saveUdeskException(exceptionRecord);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<UdeskCustomFieldOptionDTO>> getGicServiceSelect() {
        Map<String, Object> params = new HashMap<>(4);
        //枚举，取值范围：["ticket", "customer", "organization"]
        params.put("category", "ticket");
        logger.info("udesk工单gic基础服务级联链接");
        String replyRespStr = UdeskHttpClientUtil.getUdeskJsonFromParam(CUSTOM_FIELD_URL, params);
        logger.info("desk工工单gic基础服务级联返回为{}", replyRespStr);
        UdeskResCustomFieldDTO result = JSON.parseObject(replyRespStr, UdeskResCustomFieldDTO.class);
        if (result.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            //成功
            for (UdeskResCustomFieldDataDTO dataDTO : result.getFields()) {
                if ("SelectField_56661".equals(dataDTO.getField_name())) {
                    //gic基础服务级联数据
                    return ServiceResponse.success(dataDTO.getOptions());
                }
            }
        } else {
            String exceptionRecord = "udesk工单gic基础服务级联查询异常，返回为" + replyRespStr;
            logger.info(exceptionRecord);
            this.saveUdeskException(exceptionRecord);
        }
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "无数据");
    }

    @Override
    public ServiceResponse<String> ticketUploadFile(byte[] bytes, String fileName, Integer ticketId) {
        logger.info("udesk工单上传附件");
        String replyRespStr = UdeskHttpClientUtil.postUdeskJsonFromJsonForFile(UPLOAD_FILE, ticketId, bytes, fileName);
        logger.info("desk工单上传附件返回为{}", replyRespStr);
        UploadFileDTO result = JSON.parseObject(replyRespStr, UploadFileDTO.class);
        if (result.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            return ServiceResponse.success();
        }
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), result.getMessage());
    }

    private List<UdeskResCustomFieldDataDTO> queryUdeskCustomField(List<String> fieldNameList) {
        Map<String, Object> params = new HashMap<>(4);
        //枚举，取值范围：["ticket", "customer", "organization"]
        params.put("category", "ticket");
        logger.info("udesk工单自定义字段链接");
        String replyRespStr = UdeskHttpClientUtil.getUdeskJsonFromParam(CUSTOM_FIELD_URL, params);
        logger.info("desk工单自定义字段返回为{}", replyRespStr);
        UdeskResCustomFieldDTO result = JSON.parseObject(replyRespStr, UdeskResCustomFieldDTO.class);
        if (result.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            //成功
            List<UdeskResCustomFieldDataDTO> resultList = new ArrayList<>();
            for (UdeskResCustomFieldDataDTO dataDTO : result.getFields()) {
                if (fieldNameList.contains(dataDTO.getField_name())) {
                    resultList.add(dataDTO);
                }
            }
            return resultList;
        } else {
            String exceptionRecord = "udesk工单gic基础服务级联查询异常，返回为" + replyRespStr;
            logger.info(exceptionRecord);
            this.saveUdeskException(exceptionRecord);
        }
        return null;
    }

}
