package com.gic.udesk.utils;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.net.ssl.SSLContext;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.*;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.LayeredConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class UdeskHttpClientUtil {
    private static final Logger logger = LogManager.getLogger(UdeskHttpClientUtil.class);

    private static PoolingHttpClientConnectionManager connectionManager = null;

    private static CloseableHttpClient httpClient = null;

    static {
        LayeredConnectionSocketFactory sslsf = null;
        try {
            sslsf = new SSLConnectionSocketFactory(SSLContext.getDefault());
            Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                    .register("https", sslsf).register("http", new PlainConnectionSocketFactory()).build();
            connectionManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);
            connectionManager.setMaxTotal(500);
            connectionManager.setDefaultMaxPerRoute(100);
            httpClient = HttpClients.custom().setConnectionManager(connectionManager).build();
        } catch (NoSuchAlgorithmException e) {
            logger.info("加载UdeskHttpClientUtil失败", e);
        }
    }

    /**
     * udesk的get请求拼接url参数
     */
    public static String getUdeskJsonFromParam(String url, Map<String, Object> param) {
        if (param == null) {
            param = new HashMap<String, Object>();
        }
        long timeStamp = System.currentTimeMillis() / 1000;
        param.put("email", UdeskConstant.EMAIL);
        param.put("timestamp", timeStamp);
        param.put("sign", UdeskUtils.getSign(timeStamp));
        url = UdeskConstant.UDESK_URI + getURL(url, param);
        logger.info(url);
        CloseableHttpResponse response = null;
        try {
            HttpGet httpGet = new HttpGet(url);
            response = httpClient.execute(httpGet);
            return EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.info("GET请求{}失败", url, e);
        } finally {
            closeResponse(response);
        }
        return null;
    }

    /**
     * udesk的post请求，Request Body
     */
    public static String postUdeskJsonFromJson(String url, String json) {
        long timeStamp = System.currentTimeMillis() / 1000;
        url = UdeskConstant.UDESK_URI + url + "?email=" + UdeskConstant.EMAIL + "&timestamp=" + timeStamp + "&sign=" + UdeskUtils.getSign(timeStamp);
        logger.info(url);
        CloseableHttpResponse response = null;
        try {
            HttpPost httpPost = new HttpPost(url);
            response = getResponse(json, httpPost);
            return EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.info("POST请求{}失败，参数为{}", url, json, e);
        } finally {
            closeResponse(response);
        }
        return null;

    }

    /**
     * udesk的post请求，Request Body 上传文件参数天聪
     */
    public static String postUdeskJsonFromJsonForFile(String url, Integer ticketId, byte[] bytes, String fileName) {
        long timeStamp = System.currentTimeMillis() / 1000;
        url = UdeskConstant.UDESK_URI + url + "?email=" + UdeskConstant.EMAIL + "&timestamp=" + timeStamp + "&sign=" + UdeskUtils.getSign(timeStamp);
        logger.info(url);
        CloseableHttpResponse response = null;
        try {
            HttpPost httpPost = new HttpPost(url);
            response = getResponseForFile(bytes, fileName, ticketId, httpPost);
            return EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.info("POST请求{}失败，参数ticketId:{}", url, ticketId, e);
        } finally {
            closeResponse(response);
        }
        return null;

    }

    /**
     * udesk的put请求，Request Body
     */
    public static String putUdeskJsonFromJson(String url, String json) {
        long timeStamp = System.currentTimeMillis() / 1000;
        url = UdeskConstant.UDESK_URI + url + "?email=" + UdeskConstant.EMAIL + "&timestamp=" + timeStamp + "&sign=" + UdeskUtils.getSign(timeStamp);
        logger.info(url);
        CloseableHttpResponse response = null;
        try {
            HttpPut httpPut = new HttpPut(url);
            if (json != null && (!"".equals(json.trim()))) {
                StringEntity entity = new StringEntity(json, ContentType.APPLICATION_JSON);
                httpPut.setEntity(entity);
            }
            response = httpClient.execute(httpPut);
            return EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.info("PUT请求{}失败，参数为{}", url, json, e);
        } finally {
            closeResponse(response);
        }
        return null;

    }

    /**
     * udesk的delete请求拼接url参数
     */
    public static String deleteUdeskJsonFromParam(String url, Map<String, Object> param) {
        if (param == null) {
            param = new HashMap<String, Object>();
        }
        long timeStamp = System.currentTimeMillis() / 1000;
        param.put("email", UdeskConstant.EMAIL);
        param.put("timestamp", timeStamp);
        param.put("sign", UdeskUtils.getSign(timeStamp));
        url = UdeskConstant.UDESK_URI + getURL(url, param);
        logger.info(url);
        CloseableHttpResponse response = null;
        try {
            HttpDelete httpDelete = new HttpDelete(url);
            response = httpClient.execute(httpDelete);
            return EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.info("DELETE请求{}失败", url, e);
        } finally {
            closeResponse(response);
        }
        return null;
    }

    /**
     * TAPD的get请求拼接url参数
     */
    public static String getTapdJsonFromParam(String url, Map<String, Object> param) {
        CloseableHttpResponse response = null;
        try {
            url = UdeskConstant.TAPD_URI +getURIBuilder(url, param);
            logger.info(url);
            HttpGet httpGet = new HttpGet(url);
            httpGet.setHeader("Authorization", UdeskConstant.TAPD_AUTH);
            response = httpClient.execute(httpGet);
            return EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.info("GET请求{}失败", url, e);
        } finally {
            closeResponse(response);
        }
        return null;
    }

    /**
     * TAPD的post请求拼接url参数
     */
    public static String postTapdJsonFromJson(String url, String json) {
        url = UdeskConstant.TAPD_URI + url;
        logger.info(url);
        CloseableHttpResponse response = null;
        try {
            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader("Authorization", UdeskConstant.TAPD_AUTH);
            response = getResponse(json, httpPost);
            return EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.info("POST请求{}失败，参数为{}", url, json, e);
        } finally {
            closeResponse(response);
        }
        return null;
    }

    /**
     * 铁臂-使用httpclient自带的URIBuilder建立URI
     * @param uri
     * @param requestParams
     * @return String
     * @throws URISyntaxException
     */
    private static String getURIBuilder(String uri, Map<String, Object> requestParams) throws URISyntaxException {
        URIBuilder uriBuilder = new URIBuilder( uri);
        if (requestParams != null && requestParams.size() > 0) {
            for (Map.Entry<String, Object> entry : requestParams.entrySet()) {
                String key = entry.getKey();
                if (StringUtils.isNotBlank(key)) {
                    uriBuilder.setParameter(key, String.valueOf(entry.getValue()));
                }
            }
        }
        return uriBuilder.build().toString();
    }

    /**
     * 拼接URL
     */
    private static String getURL(String url, Map<String, Object> param) {
        if (param != null && param.size() > 0) {
            StringBuilder urlSB = new StringBuilder(url);
            Set<Map.Entry<String, Object>> paramSet = param.entrySet();
            boolean isFirst = true;
            for (Map.Entry<String, Object> entry : paramSet) {
                if (isFirst) {
                    urlSB.append("?");
                    isFirst = false;
                } else {
                    urlSB.append("&");
                }
                urlSB.append(entry.getKey()).append("=").append(entry.getValue());
            }
            return urlSB.toString();
        }
        return url;
    }

    /**
     * 关闭Response
     */
    private static void closeResponse(CloseableHttpResponse response) {
        if (response != null) {
            try {
                response.close();
            } catch (Exception e) {
                logger.info("关闭response失败", e);
            }
        }
    }

    /**
     * 获取Response
     */
    private static CloseableHttpResponse getResponse(String json, HttpPost httpPost)
            throws IOException, ClientProtocolException {
        CloseableHttpResponse response;
        if (json != null && (!"".equals(json.trim()))) {
            StringEntity entity = new StringEntity(json, ContentType.APPLICATION_JSON);
            httpPost.setEntity(entity);
        }
        response = httpClient.execute(httpPost);
        return response;
    }

    /**
     * 上传文件填充
     * @param bytes
     * @param fileName
     * @param ticketId
     * @param httpPost
     * @return
     * @throws IOException
     * @throws ClientProtocolException
     */
    private static CloseableHttpResponse getResponseForFile(byte[] bytes, String fileName, Integer ticketId, HttpPost httpPost)
            throws IOException, ClientProtocolException {
        CloseableHttpResponse response;
        //解决乱码，设置mode
        MultipartEntityBuilder multipartEntityBuilder = MultipartEntityBuilder.create().setMode(HttpMultipartMode.RFC6532);
        multipartEntityBuilder.addBinaryBody("file", new ByteArrayInputStream(bytes), ContentType.DEFAULT_BINARY, fileName);
        multipartEntityBuilder.addTextBody("ticket_id", ticketId.toString());
        multipartEntityBuilder.addTextBody("file_name", fileName);
        HttpEntity httpEntity = multipartEntityBuilder.build();
        httpPost.setEntity(httpEntity);
//        httpPost.setHeader("Content-Type", "application/octet-stream");
        response = httpClient.execute(httpPost);
        return response;
    }
}
