package com.gic.udesk.web.controller;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.udesk.constant.UdeskSharedConstant;
import com.gic.udesk.dto.UdeskMessageDTO;
import com.gic.udesk.service.TapdMessageApiService;
import com.gic.udesk.service.UdeskMessageApiService;

@RestController
public class MessageController {
    private static final Logger logger = LogManager.getLogger(MessageController.class);

    @Autowired
    private UdeskMessageApiService udeskMessageApiService;
    @Autowired
    private TapdMessageApiService tapdMessageApiService;

    @IgnoreLogin
    @RequestMapping("udesk-message")
    public RestResponse udeskMessage(@RequestBody UdeskMessageDTO udeskMessage) {
        logger.info("接收UDESK的消息是：{}", udeskMessage);
        udeskMessage.setSource(UdeskSharedConstant.UDESK_SOURCE);
        return ResultControllerUtils.commonResult(udeskMessageApiService.saveUdeskMessage(udeskMessage));
    }

    /**
     * 不需要同步到tapd的触发器
     * @param udeskMessage
     * @return
     */
    @IgnoreLogin
    @RequestMapping("udesk-message-no-tapd")
    public RestResponse udeskMessageNoTapd(@RequestBody UdeskMessageDTO udeskMessage) {
        logger.info("接收UDESK不需要同步tapd的消息是：{}", udeskMessage);
        udeskMessage.setSource(UdeskSharedConstant.UDESK_SOURCE);
        return ResultControllerUtils.commonResult(udeskMessageApiService.saveUdeskMessageNoTapd(udeskMessage));
    }

    @RequestMapping(value = "tapd-message", method = RequestMethod.POST)
    @IgnoreLogin
    public RestResponse tapdMessage(@RequestBody String tapdMessageStr) {
        logger.info("接收TAPD的消息:{}",tapdMessageStr);
        return ResultControllerUtils.commonResult(tapdMessageApiService.handleTapdMessage(tapdMessageStr));
    }
}
